/* 
 * $Header: /home/projects/mule/scm/mule/mule/src/java/org/mule/config/pool/CommonsPoolProxyFactory.java,v 1.1.1.1 2005/02/15 02:28:18 topping Exp $
 * $Revision: 1.1.1.1 $
 * $Date: 2005/02/15 02:28:18 $
 * ------------------------------------------------------------------------------------------------------
 * 
 * Copyright (c) Cubis Limited. All rights reserved.
 * http://www.cubis.co.uk 
 * 
 * The software in this package is published under the terms of the BSD
 * style license a copy of which has been included with this distribution in
 * the LICENSE.txt file. 
 *
 */
package org.mule.config.pool;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.pool.PoolableObjectFactory;
import org.mule.InitialisationException;
import org.mule.impl.MuleDescriptor;
import org.mule.umo.UMODescriptor;
import org.mule.util.ObjectPool;

/**
 * <code>CommonsPoolProxyFactory</code> is used to create MuleProxies for use in a proxy pool.
 * This is a jakarta commons-pool implementation.
 *
 * @author <a href="mailto:ross.mason@cubis.co.uk">Ross Mason</a>
 * @version $Revision: 1.1.1.1 $
 */
public class CommonsPoolProxyFactory extends AbstractProxyFactory implements PoolableObjectFactory
{
    /**
     * Creates a pool factory using the descriptor as the basis for creating its objects
     *
     * @param descriptor the descriptor to use to construct a MuleProxy
     * @see UMODescriptor
     */
    public CommonsPoolProxyFactory(MuleDescriptor descriptor, ObjectPool pool)
    {
        super(descriptor, pool);
    }

    /*
	 * (non-Javadoc)
	 * 
	 * @see org.apache.commons.pool.PoolableObjectFactory#activateObject(java.lang.Object)
	 */
    public void activateObject(Object arg0) throws Exception
    {

    }

    /*
	 * (non-Javadoc)
	 * 
	 * @see org.apache.commons.pool.PoolableObjectFactory#destroyObject(java.lang.Object)
	 */
    public void destroyObject(Object object) throws Exception
    {
        pool.onRemove(object);
    }

    /*
	 * (non-Javadoc)
	 * 
	 * @see org.apache.commons.pool.PoolableObjectFactory#makeObject()
	 */
    public Object makeObject() throws Exception
    {
        Object object = create();
        pool.onAdd(object);
        return object;
    }

    /*
	 * (non-Javadoc)
	 * 
	 * @see org.apache.commons.pool.PoolableObjectFactory#passivateObject(java.lang.Object)
	 */
    public void passivateObject(Object arg0) throws Exception
    {

    }

    /*
	 * (non-Javadoc)
	 * 
	 * @see org.apache.commons.pool.PoolableObjectFactory#validateObject(java.lang.Object)
	 */
    public boolean validateObject(Object arg0)
    {
        return true;
    }

    protected void afterComponentCreate(Object component) throws InitialisationException
    {
        try
        {
            BeanUtils.populate(component, descriptor.getProperties());
        }
        catch (Exception e)
        {
            throw new InitialisationException("Failed to set properties on component: " + e.getMessage(), e);
        }
    }


}
