/*
 * $Header: /home/projects/mule/scm/mule/mule-extras/groovy/src/java/org/mule/extras/groovy/GroovyComponent.java,v 1.1.1.1 2005/02/15 02:28:36 topping Exp $
 * $Revision: 1.1.1.1 $
 * $Date: 2005/02/15 02:28:36 $
 * ------------------------------------------------------------------------------------------------------
 *
 * Copyright (c) Cubis Limited. All rights reserved.
 * http://www.cubis.co.uk
 *
 * The software in this package is published under the terms of the BSD
 * style license a copy of which has been included with this distribution in
 * the LICENSE.txt file.
 */
package org.mule.extras.groovy;

import groovy.lang.GroovyClassLoader;
import groovy.lang.GroovyCodeSource;
import groovy.lang.GroovyObject;
import org.mule.InitialisationException;
import org.mule.components.script.AbstractScriptComponent;
import org.mule.umo.UMOEventContext;
import org.mule.util.ClassHelper;

import java.io.File;
import java.io.IOException;
import java.net.URL;

/**
 * <code>GroovyComponent</code> allows a grooy object ot managed as a Mule
 * component.
 *
 * @author <a href="mailto:ross.mason@cubis.co.uk">Ross Mason</a>
 * @version $Revision: 1.1.1.1 $
 */
public class GroovyComponent extends AbstractScriptComponent
{
    public static final String DEFAULT_METHOD_NAME = "onCall";
    private GroovyObject component = null;
    private String methodName = DEFAULT_METHOD_NAME;

    /**
     * Loads the script for this component
     * @param script the script file location
     * @throws InitialisationException if anything fails while starting up
     */
    protected void loadInterpreter(String script) throws InitialisationException
    {
        try
        {
            File f = new File(script);
            if(f.exists()) {
                fileChanged(f);
            } else {
                GroovyClassLoader loader = new GroovyClassLoader(Thread.currentThread().getContextClassLoader());
                URL url = ClassHelper.getResource(script, getClass());
                if(url==null) {
                    throw new InitialisationException("Could not find script: " + script);
                }
                Class groovyClass = loader.parseClass(new GroovyCodeSource(url));
                component = (GroovyObject) groovyClass.newInstance();
            }
        } catch (Exception e)
        {
            if(e instanceof InitialisationException) throw (InitialisationException)e;
            throw new InitialisationException("Failed to load groovy component: " + e.getMessage(), e);
        }
    }

    protected String getDefaultFileExtension()
    {
        return ".groovy";
    }

    /**
     * Called when one of the monitored files are created, deleted
     * or modified.
     *
     * @param file File which has been changed.
     */
    public void fileChanged(File file) throws IOException
    {
        try
        {
            GroovyClassLoader loader = new GroovyClassLoader(getClass().getClassLoader());
            Class groovyClass = loader.parseClass(new GroovyCodeSource(file));
            component = (GroovyObject) groovyClass.newInstance();
        } catch (Exception e)
        {
            throw new IOException("Failed to reload groovy script: " + e.getMessage());
        }
    }

    public String getMethodName()
    {
        return methodName;
    }

    public void setMethodName(String methodName)
    {
        this.methodName = methodName;
    }

    /**
     * Passes the context to the listener
     *
     * @param context the context ot process
     * @return Object this object can be anything. When the <code>UMOLifecycleAdapter</code> for
     *         the components receives this object it will first see if the Object is an
     *         <code>UMOEvent</code> if not and the Object is not null a new context will be created using
     *         the returned object as the payload.  This new context will then get published to the configured
     *         outbound endpoint if-
     *         <ol>
     *         <li>One has been configured for the UMO.</li>
     *         <li>the <code>setStopFurtherProcessing(true)</code> wasn't called on the previous context.</li>
     *         </ol>
     * @throws Exception if the context fails to process properly. If exceptions aren't handled by the implementation
     *                   they will be handled by the exceptionStrategy associated with the components
     */
    public Object onCall(UMOEventContext context) throws Exception
    {
        if(methodName.equals(DEFAULT_METHOD_NAME)) {
            return component.invokeMethod(getMethodName(), context);
        } else {
            Object msg = context.getTransformedMessage();
            return component.invokeMethod(getMethodName(), msg);
        }
    }
}
