/*
 * $Header: /home/projects/mule/scm/mule/mule-extras/spring/src/java/org/mule/extras/spring/config/MuleObjectNameProcessor.java,v 1.1.1.1 2005/02/15 02:28:42 topping Exp $
 * $Revision: 1.1.1.1 $
 * $Date: 2005/02/15 02:28:42 $
 * ------------------------------------------------------------------------------------------------------
 *
 * Copyright (c) Cubis Limited. All rights reserved.
 * http://www.cubis.co.uk
 *
 * The software in this package is published under the terms of the BSD
 * style license a copy of which has been included with this distribution in
 * the LICENSE.txt file.
 */
package org.mule.extras.spring.config;

import org.mule.umo.UMOAgent;
import org.mule.umo.UMODescriptor;
import org.mule.umo.endpoint.UMOEndpoint;
import org.mule.umo.model.UMOModel;
import org.mule.umo.provider.UMOConnector;
import org.mule.umo.transformer.UMOTransformer;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.config.BeanPostProcessor;

/**
 * <code>MuleObjectNameProcessor</code> is used to set spring ids to Mule object names
 * so the the bean id and name property on the object don't both have to be set.
 *
 * @author <a href="mailto:ross.mason@cubis.co.uk">Ross Mason</a>
 * @version $Revision: 1.1.1.1 $
 */

public class MuleObjectNameProcessor implements BeanPostProcessor
{
    private boolean overwrite = false;

    public Object postProcessBeforeInitialization(Object o, String s) throws BeansException
    {
        if(o instanceof UMOConnector) {
            if(((UMOConnector)o).getName()==null || overwrite) ((UMOConnector)o).setName(s);
        } else if(o instanceof UMOTransformer) {
            ((UMOTransformer)o).setName(s);
        } else if(o instanceof UMOEndpoint) {
            if(((UMOEndpoint)o).getName()==null || overwrite) ((UMOEndpoint)o).setName(s);
        } else if(o instanceof UMODescriptor) {
            if(((UMODescriptor)o).getName()==null || overwrite) ((UMODescriptor)o).setName(s);
        } else if(o instanceof UMOModel) {
            if(((UMOModel)o).getName()==null || overwrite) ((UMOModel)o).setName(s);
        }  else if(o instanceof UMOAgent) {
            if(((UMOAgent)o).getName()==null || overwrite) ((UMOAgent)o).setName(s);
        }
        return o;
    }

    public Object postProcessAfterInitialization(Object o, String s) throws BeansException
    {
        return o;
    }

    public boolean isOverwrite()
    {
        return overwrite;
    }

    public void setOverwrite(boolean overwrite)
    {
        this.overwrite = overwrite;
    }


}
