/*
 * $Header: /home/projects/mule/scm/mule/mule/src/java/org/mule/impl/InterceptorsInvoker.java,v 1.1.1.1 2005/02/15 02:27:59 topping Exp $
 * $Revision: 1.1.1.1 $
 * $Date: 2005/02/15 02:27:59 $
 * ------------------------------------------------------------------------------------------------------
 *
 * Copyright (c) Cubis Limited. All rights reserved.
 * http://www.cubis.co.uk
 *
 * The software in this package is published under the terms of the BSD
 * style license a copy of which has been included with this distribution in
 * the LICENSE.txt file.
 */
package org.mule.impl;

import org.mule.umo.Invocation;
import org.mule.umo.UMOException;
import org.mule.umo.UMOImmutableDescriptor;
import org.mule.umo.UMOInterceptor;
import org.mule.umo.UMOMessage;

import java.util.List;

/**
 * <code>InterceptorsInvoker</code> is used trigger an interceptor chain.
 *
 * @author <a href="mailto:ross.mason@cubis.co.uk">Ross Mason</a>
 * @version $Revision: 1.1.1.1 $
 */

public class InterceptorsInvoker extends Invocation
{
    private List interceptors;
    private int cursor = 0;

    public InterceptorsInvoker(List interceptors, MuleDescriptor descriptor, UMOMessage message)
    {
        this(interceptors, new ImmutableMuleDescriptor(descriptor), message);
    }

    public InterceptorsInvoker(List interceptors, UMOImmutableDescriptor descriptor, UMOMessage message)
    {
        super(descriptor, message, null);
        this.interceptors = interceptors;
    }

    public UMOMessage execute() throws UMOException
    {
        UMOMessage message = null;
        if (cursor < interceptors.size())
        {
            UMOInterceptor interceptor = (UMOInterceptor) interceptors.get(cursor);
            incCursor();
            message = interceptor.intercept(this);
            return message;
        }
        return getMessage();
    }

    private synchronized void incCursor() {
        cursor++;
    }

}
