/*
 * $Header: /home/projects/mule/scm/mule/mule/src/java/org/mule/management/mbeans/StatisticsService.java,v 1.1.1.1 2005/02/15 02:28:12 topping Exp $
 * $Revision: 1.1.1.1 $
 * $Date: 2005/02/15 02:28:12 $
 * ------------------------------------------------------------------------------------------------------
 *
 * Copyright (c) Cubis Limited. All rights reserved.
 * http://www.cubis.co.uk
 *
 * The software in this package is published under the terms of the BSD
 * style license a copy of which has been included with this distribution in
 * the LICENSE.txt file.
 */
package org.mule.management.mbeans;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.mule.MuleManager;
import org.mule.management.stats.AllStatistics;
import org.mule.management.stats.printers.CSVPrinter;
import org.mule.management.stats.printers.HtmlTablePrinter;
import org.mule.umo.UMOManager;

import java.io.StringWriter;
import java.util.Collection;

/**
 * <code>StatisicsService</code> exposes Mule processing statistics
 *
 * @author <a href="mailto:S.Vanmeerhaege@gfdi.be">Vanmeerhaeghe Stphane </a>
 * @version $Revision: 1.1.1.1 $
 */
public class StatisticsService implements StatisticsServiceMBean
{
    /**
     * logger used by this class
     */
    protected static transient Log logger = LogFactory.getLog(StatisticsService.class);

    private AllStatistics stats = new AllStatistics();
    private MuleManager manager = null;


    public void setManager(UMOManager manager)
    {
        this.manager = (MuleManager) manager;
        if (manager == null)
        {
            stats = new AllStatistics();
        } else
        {
            stats = this.manager.getStatistics();
        }

    }

    /**
     * @see org.mule.management.stats.Statistics#clear()
     */
    public void clear()
    {
        stats.clear();
    }

    /**
     * @see org.mule.management.stats.Statistics#isEnabled()
     */
    public boolean isEnabled()
    {
        return stats.isEnabled();
    }

    /**
     * @see org.mule.management.stats.Statistics#setEnabled(boolean)
     */
    public void setEnabled(boolean b)
    {
        stats.setEnabled(b);

    }

    public Collection getComponentStatistics()
    {
        return stats.getComponentStatistics();
    }

    public void logSummary()
    {
        stats.logSummary();
    }

    public void logCSVSummary()
    {
        CSVPrinter printer = new CSVPrinter(System.out);
        printer.setPrintHeaders(true);
        stats.logSummary(printer);
    }

    public String printHtmlSummary()
    {
        StringWriter w = new StringWriter();
        HtmlTablePrinter printer = new HtmlTablePrinter(w);
        stats.logSummary(printer);
        return w.toString();
    }
}