/* 
 * $Header: /home/projects/mule/scm/mule/providers/email/src/java/org/mule/providers/email/MailMessageAdapter.java,v 1.1.1.1 2005/02/15 02:27:35 topping Exp $
 * $Revision: 1.1.1.1 $
 * $Date: 2005/02/15 02:27:35 $
 * ------------------------------------------------------------------------------------------------------
 * 
 * Copyright (c) Cubis Limited. All rights reserved.
 * http://www.cubis.co.uk 
 * 
 * The software in this package is published under the terms of the BSD
 * style license a copy of which has been included with this distribution in
 * the LICENSE.txt file. 
 *
 */

package org.mule.providers.email;

import org.mule.providers.AbstractMessageAdapter;
import org.mule.umo.MessageException;
import org.mule.umo.provider.MessageTypeNotSupportedException;
import org.mule.util.Utility;

import javax.mail.Address;
import javax.mail.Header;
import javax.mail.Message;
import javax.mail.MessagingException;
import java.util.Enumeration;

/**
 * <code>MailMessageAdapter</code> is a wrapper for a javax.mail.Message.
 *
 * @author <a href="mailto:ross.mason@cubis.co.uk">Ross Mason</a>
 * @version $Revision: 1.1.1.1 $
 */
public class MailMessageAdapter extends AbstractMessageAdapter
{

    public static final String PROPERTY_TO_ADDRESSES = "toAddresses";
    public static final String PROPERTY_FROM_ADDRESS = "fromAddress";
    public static final String PROPERTY_FROM_ADDRESSES = "fromAddresses";
    public static final String PROPERTY_CC_ADDRESSES = "ccAddresses";
    public static final String PROPERTY_BCC_ADDRESSES = "bccAddresses";
    public static final String PROPERTY_SUBJECT = "subject";

    Message message = null;


    public MailMessageAdapter(Object message) throws MessageException
    {
        setMessage(message);
    }

    /* (non-Javadoc)
     * @see org.mule.providers.UMOMessageAdapter#getPayload()
     */
    public Object getPayload()
    {
        return message;
    }

    /* (non-Javadoc)
     * @see org.mule.providers.UMOMessageAdapter#getPayloadAsBytes()
     */
    public byte[] getPayloadAsBytes() throws Exception
    {
        return Utility.objectToByteArray(message.getContent());
    }


    /* (non-Javadoc)
     * @see org.mule.providers.UMOMessageAdapter#getPayloadAsString()
     */
    public String getPayloadAsString() throws Exception
    {
        return message.getContent().toString();
    }


    /* (non-Javadoc)
     * @see org.mule.providers.UMOMessageAdapter#setMessage(java.lang.Object)
     */
    private void setMessage(Object message) throws MessageException
    {
        if(message instanceof Message) {
            this.message = (Message)message;
        } else {
            throw new MessageTypeNotSupportedException(message, MailMessageAdapter.class);
        }
        //Set message attrributes as properties
        try
        {
            Address[] addresses = null;
            properties.put(PROPERTY_SUBJECT, this.message.getSubject());
            addresses = this.message.getFrom();
            if (addresses != null && addresses.length > 0)
            {
                properties.put(PROPERTY_FROM_ADDRESS, addresses[0].toString());
                properties.put(PROPERTY_FROM_ADDRESSES, addresses);
            }
            properties.put(PROPERTY_TO_ADDRESSES, this.message.getRecipients(Message.RecipientType.TO));
            properties.put(PROPERTY_CC_ADDRESSES, this.message.getRecipients(Message.RecipientType.CC));
            properties.put(PROPERTY_BCC_ADDRESSES, this.message.getRecipients(Message.RecipientType.BCC));

            for (Enumeration e = this.message.getAllHeaders(); e.hasMoreElements();)
            {
                Header h = (Header)e.nextElement();
                properties.put(h.getName(), h.getValue());
            }

        }
        catch (MessagingException e)
        {
            throw new MessageException("Failed to create message adapter for message: " + e, e);
        }
    }
}
