/*
 * $Header: /home/projects/mule/scm/mule/providers/soap/src/java/org/mule/providers/soap/axis/AxisMessageAdapter.java,v 1.1.1.1 2005/02/15 02:27:32 topping Exp $
 * $Revision: 1.1.1.1 $
 * $Date: 2005/02/15 02:27:32 $
 * ------------------------------------------------------------------------------------------------------
 *
 * Copyright (c) Cubis Limited. All rights reserved.
 * http://www.cubis.co.uk
 *
 * The software in this package is published under the terms of the BSD
 * style license a copy of which has been included with this distribution in
 * the LICENSE.txt file.
 */
package org.mule.providers.soap.axis;

import org.apache.axis.MessageContext;
import org.mule.providers.AbstractMessageAdapter;
import org.mule.transformers.simple.SerialisableToByteArray;
import org.mule.umo.MessageException;
import org.mule.umo.transformer.UMOTransformer;

import javax.xml.soap.SOAPException;

/**
 * <code>AxisMessageAdapter</code> TODO
 *
 * @author <a href="mailto:ross.mason@cubis.co.uk">Ross Mason</a>
 * @version $Revision: 1.1.1.1 $
 */
public class AxisMessageAdapter extends AbstractMessageAdapter
{
    private Object message;
    private UMOTransformer trans = new SerialisableToByteArray();

    public AxisMessageAdapter(Object message) throws MessageException
    {
        this.message = message;
        try
        {
            MessageContext ctx = MessageContext.getCurrentContext();
            if(ctx!=null)
            {
                MuleSoapHeaders header = new MuleSoapHeaders(ctx.getMessage().getSOAPPart().getEnvelope().getHeader());

                if(header.getReplyTo()!=null && !"".equals(header.getReplyTo())) {
                    setReplyTo(header.getReplyTo());
                }

                if(header.getCorrelationGroup()!=null && !"".equals(header.getCorrelationGroup())
                        && !"-1".equals(header.getCorrelationGroup())) {
                    setCorrelationGroupSize(Integer.parseInt(header.getCorrelationGroup()));
                }
                if(header.getCorrelationSequence()!=null && !"".equals(header.getCorrelationSequence())
                        && !"-1".equals(header.getCorrelationSequence())) {
                    setCorrelationSequence(Integer.parseInt(header.getCorrelationSequence()));
                }
                if(header.getCorrelationId()!=null && !"".equals(header.getCorrelationId())) {
                    setCorrelationId(header.getCorrelationId());
                }
            }
        } catch (SOAPException e)
        {
            throw new MessageException("Failed to process Soap Header: " + e.getMessage(), e);
        }
    }

    /**
     * Converts the message implementation into a String representation
     *
     * @return String representation of the message payload
     * @throws Exception Implementation may throw an endpoint specific exception
     */
    public String getPayloadAsString() throws Exception
    {
        return new String(getPayloadAsBytes());
    }

    /**
     * Converts the message implementation into a String representation
     *
     * @return String representation of the message
     * @throws Exception Implemetation may throw an endpoint specific exception
     */
    public byte[] getPayloadAsBytes() throws Exception
    {
        return (byte[])trans.transform(message);
    }

    /**
     * @return the current message
     */
    public Object getPayload()
    {
        return message;
    }
}