/*
 * $Header: /home/projects/mule/scm/mule/providers/soap/src/java/org/mule/providers/soap/glue/GlueMessageAdapter.java,v 1.1.1.1 2005/02/15 02:27:31 topping Exp $
 * $Revision: 1.1.1.1 $
 * $Date: 2005/02/15 02:27:31 $
 * ------------------------------------------------------------------------------------------------------
 *
 * Copyright (c) Cubis Limited. All rights reserved.
 * http://www.cubis.co.uk
 *
 * The software in this package is published under the terms of the BSD
 * style license a copy of which has been included with this distribution in
 * the LICENSE.txt file.
 */
package org.mule.providers.soap.glue;

import electric.glue.context.ThreadContext;
import electric.service.IService;
import org.mule.config.MuleProperties;
import org.mule.providers.AbstractMessageAdapter;
import org.mule.transformers.simple.SerialisableToByteArray;
import org.mule.umo.transformer.UMOTransformer;
import org.mule.util.IteratorAdapter;

/**
 * <code>GlueMessageAdapter</code> wraps a Glue soap request
 *
 * @author <a href="mailto:ross.mason@cubis.co.uk">Ross Mason</a>
 * @version $Revision: 1.1.1.1 $
 */
public class GlueMessageAdapter extends AbstractMessageAdapter
{
    private Object message;
    private UMOTransformer trans = new SerialisableToByteArray();

    public GlueMessageAdapter(Object message)
    {
        if(message instanceof GlueMessageHolder) {
            GlueMessageHolder holder = (GlueMessageHolder) message;
            this.message = holder.getMessage();
            IteratorAdapter iter = new IteratorAdapter(holder.getService().getContext().getPropertyNames());
            String key;
            while(iter.hasNext()) {
                key = iter.next().toString();
                setProperty(key, holder.getService().getContext().getProperty(key));
            }
        } else {
            this.message = message;
        }
        String value = (String)ThreadContext.removeProperty(MuleProperties.MULE_REPLY_TO_PROPERTY);
        if(value!=null) setReplyTo(value);
        value = (String)ThreadContext.removeProperty(MuleProperties.MULE_CORRELATION_ID_PROPERTY);
        if(value!=null) setCorrelationId(value);

        value = (String)ThreadContext.removeProperty(MuleProperties.MULE_CORRELATION_SEQUENCE_PROPERTY);
        if(value !=null && !"-1".equals(value)) setCorrelationSequence(Integer.parseInt(value));
        value = (String)ThreadContext.removeProperty(MuleProperties.MULE_CORRELATION_GROUP_SIZE_PROPERTY);
        if(value !=null && !"-1".equals(value)) setCorrelationGroupSize(Integer.parseInt(value));
    }

    /**
     * Converts the message implementation into a String representation
     *
     * @return String representation of the message payload
     * @throws Exception Implementation may throw an endpoint specific exception
     */
    public String getPayloadAsString() throws Exception
    {
        return new String(getPayloadAsBytes());
    }

    /**
     * Converts the message implementation into a String representation
     *
     * @return String representation of the message
     * @throws Exception Implemetation may throw an endpoint specific exception
     */
    public byte[] getPayloadAsBytes() throws Exception
    {
        return (byte[])trans.transform(message);
    }

    /**
     * @return the current message
     */
    public Object getPayload()
    {
        return message;
    }

    public static class GlueMessageHolder
    {
        private Object message;
        private IService service;

        public GlueMessageHolder(Object message, IService service)
        {
            this.message = message;
            this.service = service;
        }

        public Object getMessage()
        {
            return message;
        }

        public IService getService()
        {
            return service;
        }
    }
}
