/*
 * $Header: /home/projects/mule/scm/mule/providers/udp/src/java/org/mule/providers/udp/UdpMessageDispatcher.java,v 1.1.1.1 2005/02/15 02:27:34 topping Exp $
 * $Revision: 1.1.1.1 $
 * $Date: 2005/02/15 02:27:34 $
 * ------------------------------------------------------------------------------------------------------
 *
 * Copyright (c) Cubis Limited. All rights reserved.
 * http://www.cubis.co.uk
 *
 * The software in this package is published under the terms of the BSD
 * style license a copy of which has been included with this distribution in
 * the LICENSE.txt file.
 */
package org.mule.providers.udp;

import EDU.oswego.cs.dl.util.concurrent.SynchronizedBoolean;
import org.mule.MuleManager;
import org.mule.impl.MuleMessage;
import org.mule.providers.AbstractMessageDispatcher;
import org.mule.umo.UMOEvent;
import org.mule.umo.UMOException;
import org.mule.umo.UMOMessage;
import org.mule.umo.endpoint.UMOEndpointURI;
import org.mule.umo.provider.UMOConnector;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.URI;
import java.net.URISyntaxException;

/**
 * <code>UdpMessageDispatcher</code> TODO
 *
 * @author <a href="mailto:ross.mason@cubis.co.uk">Ross Mason</a>
 * @version $Revision: 1.1.1.1 $
 */

public class UdpMessageDispatcher extends AbstractMessageDispatcher
{
    protected UdpConnector connector;
    protected InetAddress inetAddress;
    protected DatagramSocket socket;
    protected int port;
    protected SynchronizedBoolean initialised = new SynchronizedBoolean(false);

    public UdpMessageDispatcher(UdpConnector connector)
    {
        super(connector);
        this.connector = connector;
    }

    protected DatagramSocket createSocket(int port, InetAddress inetAddress) throws IOException
    {
        return new DatagramSocket(port, inetAddress);
    }

    protected void initialise(String endpoint) throws IOException, URISyntaxException
    {
        if (!initialised.get())
        {
            URI uri = new URI(endpoint);
            port = uri.getPort();
            inetAddress = InetAddress.getByName(uri.getHost());
            socket = createSocket(port, inetAddress);
            socket.setReceiveBufferSize(connector.getBufferSize());
            socket.setSendBufferSize(connector.getBufferSize());
            socket.setSoTimeout(connector.getTimeout());
            initialised.set(true);
        }
    }

    public void doDispatch(UMOEvent event) throws Exception
    {
        initialise(event.getEndpoint().getEndpointURI().getAddress());
        byte[] payload = event.getTransformedMessageAsBytes();
        write(socket, payload);
    }

    protected void write(DatagramSocket socket, byte[] data) throws IOException
    {
        DatagramPacket packet = new DatagramPacket(data, data.length);
        if (port >= 0)
        {
            packet.setPort(port);
        }
        packet.setAddress(inetAddress);
        socket.send(packet);
    }

    public UMOMessage doSend(UMOEvent event) throws Exception
    {
        doDispatch(event);
        //If we're doing sync receive try and read return info from socket
        if (MuleManager.getConfiguration().isSynchronousReceive())
        {
            DatagramPacket result = receive(socket, connector.getTimeout());
            if (result == null) return null;
            UMOMessage message = new MuleMessage(connector.getMessageAdapter(result));
            return message;
        } else
        {
            return event.getMessage();
        }
    }

    private DatagramPacket receive(DatagramSocket socket, int timeout) throws IOException
    {
        int origTimeout = socket.getSoTimeout();
        try
        {
            DatagramPacket packet = new DatagramPacket(new byte[connector.getBufferSize()], connector.getBufferSize());
            socket.setSoTimeout(timeout);
            socket.receive(packet);
            return packet;
        } finally
        {
            socket.setSoTimeout(origTimeout);
        }
    }

    public UMOMessage receive(UMOEndpointURI endpointUri, long timeout) throws Exception
    {
        initialise(endpointUri.getAddress());
        DatagramPacket result = receive(socket, Integer.parseInt(String.valueOf(timeout)));
        if (result == null) return null;
        UMOMessage message = new MuleMessage(connector.getMessageAdapter(result));
        return message;
    }

    public Object getDelegateSession() throws UMOException
    {
        return null;
    }

    public UMOConnector getConnector()
    {
        return connector;
    }

    public void doDispose() throws UMOException
    {
        initialised.set(false);
        if(socket!=null) socket.close();
    }
}
