/*
 * $Header: /home/projects/mule/scm/mule/mule/src/java/org/mule/routing/filters/WildcardFilter.java,v 1.1.1.1 2005/02/15 02:27:56 topping Exp $
 * $Revision: 1.1.1.1 $
 * $Date: 2005/02/15 02:27:56 $
 * ------------------------------------------------------------------------------------------------------
 *
 * Copyright (c) Cubis Limited. All rights reserved.
 * http://www.cubis.co.uk
 *
 * The software in this package is published under the terms of the BSD
 * style license a copy of which has been included with this distribution in
 * the LICENSE.txt file.
 */
package org.mule.routing.filters;

import org.mule.umo.UMOFilter;
import org.mule.util.Utility;

/**
 * <code>WildcardFilter</code> is used to match wildcard string.  It performs
 * matches with * i.e.
 *
 * jms.events.* would catch
 * jms.events.customer
 * jms.events.receipts
 *
 * This filter accepts a comma separented list of patterns so more than one
 * filter pattenr can be matched for a given argument i.e.-
 *
 * jms.events.*, jms.actions.*
 *
 * will match jms.events.system and jms.actions but not jms.queue 
 *
 * @author <a href="mailto:ross.mason@cubis.co.uk">Ross Mason</a>
 * @version $Revision: 1.1.1.1 $
 */

public class WildcardFilter implements UMOFilter
{
    protected String[] patterns;
    protected String pattern;

    public WildcardFilter()
    {
    }


    public WildcardFilter(String pattern) {
        setPattern(pattern);
    }

    public boolean accept(Object object)
    {
        if(object==null) return false;
        String pattern = null;
        boolean match = false;
        for (int x = 0; x < patterns.length; x++)
        {
            pattern = patterns[x];

            String string = object.toString();
            if("*".equals(pattern) || "**".equals(pattern)) {
                return true;
            }
            int i = pattern.indexOf("*");

            if(i == -1) {
                match = pattern.equals(string);
            } else {
                int i2 = pattern.indexOf("*", i + 1);
                if(i2 > 1) {
                    match = string.indexOf(pattern.substring(1, i2)) > -1;
                }else if (i == 0) {
                    match = string.endsWith(pattern.substring(1));
                } else {
                    match = string.startsWith(pattern.substring(0, i));
                }
            }
            if(match) return true;
        }
        return false;
    }

    public String getPattern()
    {
        return pattern;
    }

    public void setPattern(String pattern)
    {
        this.pattern = pattern;
        this.patterns = Utility.split(pattern, ",");
    }
}
