/*
 * $Header: /home/projects/mule/scm/mule/mule/src/java/org/mule/routing/inbound/AbstractEventResequencer.java,v 1.1.1.1 2005/02/15 02:27:56 topping Exp $
 * $Revision: 1.1.1.1 $
 * $Date: 2005/02/15 02:27:56 $
 * ------------------------------------------------------------------------------------------------------
 *
 * Copyright (c) Cubis Limited. All rights reserved.
 * http://www.cubis.co.uk
 *
 * The software in this package is published under the terms of the BSD
 * style license a copy of which has been included with this distribution in
 * the LICENSE.txt file.
 */
package org.mule.routing.inbound;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.mule.umo.UMOEvent;
import org.mule.umo.routing.RoutingException;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <code>AbstractEventResequencer</code> is used to receive a set of events, resequence them and forward
 * them on to their destination
 *
 * @author <a href="mailto:ross.mason@cubis.co.uk">Ross Mason</a>
 * @version $Revision: 1.1.1.1 $
 */

public abstract class AbstractEventResequencer extends SelectiveConsumer
{
    protected static final String NO_CORRELATION_ID = "no-id";

    /**
     * logger used by this class
     */
    protected static transient Log logger = LogFactory.getLog(AbstractEventResequencer.class);


    private Comparator comparator;
    private Map eventGroups = new HashMap();

    public UMOEvent[] process(UMOEvent event) throws RoutingException
    {
        if(isMatch(event)) {
            EventGroup eg = addEvent(event);
            if(shouldResequence(eg)) {
                List resequencedEvents = resequenceEvents(eg);
                removeGroup(eg.getGroupId());
                UMOEvent[] returnEvents = new UMOEvent[resequencedEvents.size()];
                resequencedEvents.toArray(returnEvents);
                return returnEvents;
            }
        }
        return null;
    }

    protected EventGroup addEvent(UMOEvent event) {
        String cId = event.getMessage().getCorrelationId();
        if(cId==null) cId = NO_CORRELATION_ID;
        EventGroup eg = (EventGroup) eventGroups.get(cId);
        if(eg==null) {
            eg = new EventGroup(cId);
            eg.addEvent(event);
            eventGroups.put(eg.getGroupId(), eg);
        } else {
            eg.addEvent(event);
        }
        return eg;
    }

    protected void removeGroup(String id)
    {
        eventGroups.remove(id);
    }

    protected List resequenceEvents(EventGroup events){
        List result = new ArrayList(events.getEvents());
        if(comparator!=null) {
            Collections.sort(result, comparator);
        } else {
            logger.warn("Event comparator is null, events were not reordered");
        }
        return result;
    }

    public Comparator getComparator()
    {
        return comparator;
    }

    public void setComparator(Comparator comparator)
    {
        this.comparator = comparator;
    }

    protected abstract boolean shouldResequence(EventGroup events);
}
