/*
 * $Header: /home/projects/mule/scm/mule/mule/src/java/org/mule/routing/outbound/FilteringOutboundRouter.java,v 1.1.1.1 2005/02/15 02:27:55 topping Exp $
 * $Revision: 1.1.1.1 $
 * $Date: 2005/02/15 02:27:55 $
 * ------------------------------------------------------------------------------------------------------
 *
 * Copyright (c) Cubis Limited. All rights reserved.
 * http://www.cubis.co.uk
 *
 * The software in this package is published under the terms of the BSD
 * style license a copy of which has been included with this distribution in
 * the LICENSE.txt file.
 */
package org.mule.routing.outbound;

import org.mule.routing.filters.MessageFilter;
import org.mule.umo.UMOException;
import org.mule.umo.UMOFilter;
import org.mule.umo.UMOMessage;
import org.mule.umo.UMOSession;
import org.mule.umo.endpoint.UMOEndpoint;
import org.mule.umo.routing.CouldNotRouteOutboundMessageException;
import org.mule.umo.routing.RoutingException;
import org.mule.umo.transformer.UMOTransformer;

/**
 * <code>FilteringRouter</code> is a router that accepts events based on a filter set.
 *
 * @author <a href="mailto:ross.mason@cubis.co.uk">Ross Mason</a>
 * @version $Revision: 1.1.1.1 $
 */

public class FilteringOutboundRouter extends AbstractOutboundRouter
{
    private UMOTransformer transformer;

    private UMOFilter filter;

    public UMOMessage route(UMOMessage message, UMOSession session, boolean synchronous) throws RoutingException
    {
        UMOMessage result = null;
        if (endpoints == null || endpoints.size() == 0)
        {
            throw new RoutingException("No endpoints are set on this router, cannot route message");
        }
        try
        {

            if (synchronous)
            {
                result = send(session, message, (UMOEndpoint) endpoints.get(0));
            } else
            {
                dispatch(session, message, (UMOEndpoint) endpoints.get(0));
            }
        } catch (UMOException e)
        {
            throw new CouldNotRouteOutboundMessageException(e.getMessage(), e, message);
        }
        return result;
    }

    public UMOFilter getFilter()
    {
        return filter;
    }

    public void setFilter(UMOFilter filter)
    {
        this.filter = filter;
    }

    public boolean isMatch(UMOMessage message) throws RoutingException
    {
        if(getFilter()==null) return true;
        if(getFilter() instanceof MessageFilter) {
            return getFilter().accept(message);
        }
        try
        {
            Object payload = message.getPayload();

            if (transformer != null)
            {
                payload = transformer.transform(message);
            }
            return getFilter().accept(payload);

        } catch (Exception e)
        {
            throw new RoutingException("Failed to transform event payload: " + e, e);
        }
    }

    public UMOTransformer getTransformer()
    {
        return transformer;
    }

    public void setTransformer(UMOTransformer transformer)
    {
        this.transformer = transformer;
    }
}
