/*
 * $Header:
 * /cvsroot/mule/mule/src/test/org/mule/test/mule/AbstractMuleTestCase.java,v
 * 1.7 2003/11/24 09:58:47 rossmason Exp $ $Revision: 1.1.1.1 $ $Date: 2003/11/24
 * 09:58:47 $
 * ------------------------------------------------------------------------------------------------------
 * 
 * Copyright (c) Cubis Limited. All rights reserved. http://www.cubis.co.uk
 * 
 * The software in this package is published under the terms of the BSD style
 * license a copy of which has been included with this distribution in the
 * LICENSE.txt file.
 *  
 */

package org.mule.tck;

import com.mockobjects.dynamic.Mock;
import org.mule.MuleException;
import org.mule.MuleManager;
import org.mule.config.PoolingProfile;
import org.mule.impl.DefaultExceptionStrategy;
import org.mule.impl.MuleComponent;
import org.mule.impl.MuleDescriptor;
import org.mule.impl.MuleEvent;
import org.mule.impl.MuleMessage;
import org.mule.impl.MuleModel;
import org.mule.impl.MuleSession;
import org.mule.impl.endpoint.MuleEndpoint;
import org.mule.impl.endpoint.MuleEndpointURI;
import org.mule.tck.testmodels.mule.TestCompressionTransformer;
import org.mule.tck.testmodels.mule.TestConnector;
import org.mule.umo.UMOComponent;
import org.mule.umo.UMODescriptor;
import org.mule.umo.UMOEvent;
import org.mule.umo.UMOException;
import org.mule.umo.UMOManager;
import org.mule.umo.UMOSession;
import org.mule.umo.UMOTransaction;
import org.mule.umo.UMOTransactionFactory;
import org.mule.umo.endpoint.UMOEndpoint;
import org.mule.umo.provider.UMOConnector;
import org.mule.umo.provider.UMOMessageDispatcher;
import org.mule.umo.transformer.UMOTransformer;
import org.mule.util.ClassHelper;

/**
 * <code>AbstractMuleTestCase</code> is a base class for Mule testcases.  This
 * implementation provides services to test code for creating mock and test objects.
 *
 * @author <a href="mailto:ross.mason@cubis.co.uk">Ross Mason</a>
 * @version $Revision: 1.1.1.1 $
 */
public abstract class AbstractMuleTestCase extends NamedTestCase
{
    public AbstractMuleTestCase()
    {
        super();
    }

    public static UMOManager getManager() throws Exception
    {
        UMOManager manager;
        if(MuleManager.isInstanciated()) MuleManager.getInstance().dispose();
        manager = MuleManager.getInstance();
        manager.setModel(new MuleModel());
        MuleManager.getConfiguration().setSynchronous(true);
        MuleManager.getConfiguration().getPoolingProfile().setInitialisationPolicy(PoolingProfile.POOL_INITIALISE_NO_COMPONENTS);
        return manager;
    }

    public static UMOEndpoint getTestEndpoint(String name, String type) throws UMOException
    {
        UMOEndpoint endpoint = new MuleEndpoint();
        //need to build endpoint this way to avoid depenency to any endpoint jars
        UMOConnector connector = null;
        try
        {
            connector = (UMOConnector) ClassHelper.loadClass("org.mule.tck.testmodels.mule.TestConnector", AbstractMuleTestCase.class).newInstance();
        }
        catch (Exception e)
        {
            throw new MuleException("Failed to create endpoint: " + e.getMessage(), e);
        }
        connector.setName("testConnector");
        endpoint.setConnector(connector);
        endpoint.setEndpointURI(new MuleEndpointURI("test://test"));
        endpoint.setName(name);
        endpoint.setType(type);
        return endpoint;
    }


    public static UMOEvent getTestEvent(Object data) throws UMOException
    {
        UMOComponent component = getTestComponent(getTestDescriptor("string", String.class.getName()));
        UMOSession session = getTestSession(component);
        UMOEvent event = new MuleEvent(new MuleMessage(data, null), getTestEndpoint("test1", UMOEndpoint.ENDPOINT_TYPE_SENDER), session, true);
        return event;
    }

    public static UMOTransformer getTestTransformer()
    {
        return new TestCompressionTransformer();
    }

    public static UMOEvent getTestEvent(Object data, MuleDescriptor descriptor) throws UMOException
    {
        UMOComponent component = getTestComponent(descriptor);

        UMOSession session = getTestSession(component);
        UMOEvent event = new MuleEvent(new MuleMessage(data, null), getTestEndpoint("test1", UMOEndpoint.ENDPOINT_TYPE_SENDER), getTestSession(component), true);
        return event;
    }

    public static UMOEvent getTestEvent(Object data, UMOEndpoint endpoint) throws UMOException
    {
        UMOSession session = getTestSession(getTestComponent(getTestDescriptor("string", String.class.getName())));
        UMOEvent event = new MuleEvent(new MuleMessage(data, null), endpoint, session, true);
        return event;
    }

     public static UMOEvent getTestEvent(Object data, MuleDescriptor descriptor, UMOEndpoint endpoint) throws UMOException
    {
        UMOSession session = getTestSession(getTestComponent(descriptor));
        UMOEvent event = new MuleEvent(new MuleMessage(data, null), endpoint, session, true);
        return event;
    }

    public static UMOSession getTestSession(UMOComponent component)
    {
        return new MuleSession((MuleComponent) component, null);
    }

    public static TestConnector getTestConnector()
    {
        return new TestConnector();
    }

    public static MuleComponent getTestComponent(MuleDescriptor descriptor)
    {
        return new MuleComponent(descriptor);
    }

    public static MuleDescriptor getTestDescriptor(String name, String implementation) throws UMOException
    {
        MuleDescriptor descriptor = new MuleDescriptor();
        descriptor.setExceptionStrategy(new DefaultExceptionStrategy());
        descriptor.setName(name);
        descriptor.setImplementation(implementation);
        descriptor.setOutboundEndpoint(getTestEndpoint("test1", UMOEndpoint.ENDPOINT_TYPE_SENDER));
        descriptor.initialise();
             
        descriptor.getPoolingProfile().setInitialisationPolicy(PoolingProfile.POOL_INITIALISE_NO_COMPONENTS);

        return descriptor;
    }

    public static UMOManager getTestManager()
    {
        return MuleManager.getInstance();
    }

    public static Mock getMockSession()
    {
        return new Mock(UMOSession.class, "umoSession");
    }

    public static Mock getMockMessageDispatcher()
    {
        return new Mock(UMOMessageDispatcher.class, "umoConnectorSession");
    }

    public static Mock getMockConnector()
    {
        return new Mock(UMOConnector.class, "umoConnector");
    }

    public static Mock getMockEvent()
    {
        return new Mock(UMOEvent.class, "umoEvent");
    }

    public static Mock getMockManager()
    {
        return new Mock(MuleManager.class, "muleManager");
    }

    public static Mock getMockEndpoint()
    {
        return new Mock(UMOEndpoint.class, "umoEndpoint");
    }

    public static Mock getMockDescriptor()
    {
        return new Mock(UMODescriptor.class, "umoDescriptor");
    }

    public static Mock getMockTransaction()
    {
        return new Mock(UMOTransaction.class, "umoTransaction");
    }

    public static Mock getMockTransactionFactory()
    {
        return new Mock(UMOTransactionFactory.class, "umoTransactionFactory");
    }
}
