/* 
 * $Header: /home/projects/mule/scm/mule/mule/src/java/org/mule/umo/UMOComponent.java,v 1.1.1.1 2005/02/15 02:28:21 topping Exp $
 * $Revision: 1.1.1.1 $
 * $Date: 2005/02/15 02:28:21 $
 * ------------------------------------------------------------------------------------------------------
 * 
 * Copyright (c) Cubis Limited. All rights reserved.
 * http://www.cubis.co.uk 
 * 
 * The software in this package is published under the terms of the BSD
 * style license a copy of which has been included with this distribution in
 * the LICENSE.txt file. 
 *
 */
package org.mule.umo;

import org.mule.umo.lifecycle.Initialisable;
import org.mule.umo.lifecycle.Lifecycle;

import java.io.Serializable;

/**
 * <code>UMOComponent</code> is the interal repesentation of a Mule Managed component.
 * It is responsible for managing the interaction of events to and from the component
 * as well as managing pooled resources.
 *
 * @author <a href="mailto:ross.mason@cubis.co.uk">Ross Mason</a>
 * @version $Revision: 1.1.1.1 $
 */

public interface UMOComponent extends Runnable, Serializable, Lifecycle, Initialisable
{
    /**
     * @return the UMODescriptor associated with the component
     * @see UMODescriptor
     */
    public UMODescriptor getDescriptor();

    /**
     * Makes an asynhronous event call to the component.
     * @param event the event to consume
     * @throws UMOException if the event fails to be processed
     */
    public void dispatchEvent(UMOEvent event) throws UMOException;

    /**
     * Makes a synhronous event call to the component.  This event will be
     * consumed by the component and a result returned.
     * @param event the event to consume
     * @return a UMOMessage containing the resulting message and properties
     * @throws UMOException if the event fails to be processed
     */
    public UMOMessage sendEvent(UMOEvent event) throws UMOException;

    /**
     * Pauses event processing for theComponent.
     * Unlike stop(), a paused component will still consume messages from the
     * underlying transport, but those messages will be queued until the component is
     * resumed.
     *
     * In order to persist these queued messages you can set the 'recoverableMode' property
     * on the Muleconfiguration to true.  this causes all internal queues to store their state.
     *
     * @throws UMOException if the component failed to pause.
     * @see org.mule.config.MuleConfiguration
     */
    public void pause() throws UMOException;

    /**
     * Resumes the Component that has been paused. If the component is not paused
     * nothing is executed.
     * @throws UMOException if the component failed to resume
     */
    public void resume() throws UMOException;
}
