/*
 * $Header: /home/projects/mule/scm/mule/providers/multicast/src/test/java/org/mule/providers/multicast/MulticastConnectorFunctionalTestCase.java,v 1.1 2005/02/16 23:03:32 gnt Exp $
 * $Revision: 1.1 $
 * $Date: 2005/02/16 23:03:32 $
 * ------------------------------------------------------------------------------------------------------
 *
 * Copyright (c) Cubis Limited. All rights reserved.
 * http://www.cubis.co.uk
 *
 * The software in this package is published under the terms of the BSD
 * style license a copy of which has been included with this distribution in
 * the LICENSE.txt file.
 */
package org.mule.providers.multicast;

import org.mule.impl.endpoint.MuleEndpointURI;
import org.mule.providers.udp.UdpMessageAdapter;
import org.mule.tck.functional.AbstractProviderFunctionalTestCase;
import org.mule.umo.endpoint.MalformedEndpointException;
import org.mule.umo.endpoint.UMOEndpointURI;
import org.mule.umo.provider.UMOConnector;

import java.net.DatagramPacket;
import java.net.InetAddress;
import java.net.MulticastSocket;
import java.net.URI;

/**
 * @author <a href="mailto:ross.mason@cubis.co.uk">Ross Mason</a>
 * @version $Revision: 1.1 $
 */

public class MulticastConnectorFunctionalTestCase extends AbstractProviderFunctionalTestCase
{
    private MulticastSocket s = null;
    private InetAddress inet = null;
    private URI uri;


    protected void setUp() throws Exception
    {
        super.setUp();
        uri = getInDest().getUri();
        inet = InetAddress.getByName(uri.getHost());

    }

    protected void tearDown() throws Exception
    {
        try
        {
            s.close();
        } catch (Exception e)
        {
        }
        super.tearDown();
    }

//    public void testX() throws Exception
//    {
//        String msg = "Hello";
//        InetAddress group = InetAddress.getByName("228.5.6.7");
//        MulticastSocket s = new MulticastSocket(6789);
//        s.joinGroup(group);
//        DatagramPacket hi = new DatagramPacket(msg.getBytes(), msg.length(),
//                group, 6789);
//        s.send(hi);
//        // get their responses!
//        byte[] buf = new byte[1000];
//        DatagramPacket recv = new DatagramPacket(buf, buf.length);
//        s.setBroadcast(true);
//        s.receive(recv);
//        // OK, I'm done talking - leave the group...
//        System.out.println(new String(recv.getData()));
//        s.leaveGroup(group);
//    }

    protected void sendTestData(int iterations) throws Exception
    {

        s = new MulticastSocket(uri.getPort());
        s.joinGroup(inet);
        for (int i = 0; i < iterations; i++)
        {
            String msg = "Hello" + i;

            DatagramPacket packet = new DatagramPacket(msg.getBytes(), msg.length(), inet, uri.getPort());

            System.out.println("sending message: " + i);
            s.send(packet);
        }
    }

    protected void receiveAndTestResults() throws Exception
    {
        int i = 0;
//        URI uri = new URI(getOutDest());
//        InetAddress inet = InetAddress.getByName(uri.getHost());
        MulticastSocket s = new MulticastSocket(uri.getPort());
        try
        {
            s.joinGroup(inet);
            s.setSoTimeout(2000);
            for (i = 0; i < 100; i++)
            {

                DatagramPacket packet = new DatagramPacket(new byte[32], 32, inet, uri.getPort());

                s.receive(packet);
                UdpMessageAdapter adapter = new UdpMessageAdapter(packet);
                System.out.println("Received message: " + adapter.getPayloadAsString());

            }
            assertEquals(100, i);
        } finally
        {
            s.leaveGroup(inet);
            s.close();
        }
    }

    protected UMOEndpointURI getInDest()
    {
        try
        {
            return new MuleEndpointURI("multicast://228.8.9.10:6677");
        } catch (MalformedEndpointException e)
        {
            fail(e.getMessage());
            return null;
        }
    }

    protected UMOEndpointURI getOutDest()
    {
        return getInDest();
    }

    public UMOConnector createConnector() throws Exception
    {
        MulticastConnector connector = new MulticastConnector();
        connector.setName("testMulticast");
        connector.getDispatcherThreadingProfile().setDoThreading(false);
        
        connector.setBufferSize(1024);
        return connector;
    }
}