/*
 * $Header: /home/projects/mule/scm/mule/providers/email/src/test/java/org/mule/test/providers/email/transformers/MailMessageTransformersTestCase.java,v 1.1 2005/02/16 22:58:23 gnt Exp $
 * $Revision: 1.1 $
 * $Date: 2005/02/16 22:58:23 $
 * ------------------------------------------------------------------------------------------------------
 *
 * Copyright (c) Cubis Limited. All rights reserved.
 * http://www.cubis.co.uk
 *
 * The software in this package is published under the terms of the BSD
 * style license a copy of which has been included with this distribution in
 * the LICENSE.txt file.
 */
package org.mule.test.providers.email.transformers;

import com.mockobjects.dynamic.Mock;
import org.mule.impl.endpoint.MuleEndpoint;
import org.mule.providers.email.SmtpConnector;
import org.mule.providers.email.transformers.EmailMessageToString;
import org.mule.providers.email.transformers.StringToEmailMessage;
import org.mule.tck.AbstractTransformerTestCase;
import org.mule.umo.UMOException;
import org.mule.umo.endpoint.UMOEndpoint;
import org.mule.umo.provider.UMOMessageDispatcher;
import org.mule.umo.transformer.UMOTransformer;

import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Session;
import javax.mail.internet.MimeMessage;
import java.util.Properties;

/**
 * @author <a href="mailto:ross.mason@cubis.co.uk">Ross Mason</a>
 * @version $Revision: 1.1 $
 */

public class MailMessageTransformersTestCase extends AbstractTransformerTestCase
{
    private Message message;

    public UMOTransformer getTransformer() throws Exception
    {
        return new EmailMessageToString();
    }

    public UMOTransformer getRoundTripTransformer() throws Exception
    {
        StringToEmailMessage trans = new StringToEmailMessage();
        UMOEndpoint endpoint = new MuleEndpoint("smtp://a:a@a.com", false);
        final Mock mockDispatcher = new Mock(UMOMessageDispatcher.class);
        mockDispatcher.expectAndReturn("getDelegateSession", Session.getDefaultInstance(new Properties()));
        mockDispatcher.expectAndReturn("getDelegateSession", Session.getDefaultInstance(new Properties()));

        endpoint.setConnector(new SmtpConnector() {
            public UMOMessageDispatcher getDispatcher() throws UMOException
            {
                return (UMOMessageDispatcher)mockDispatcher.proxy();
            }
        });
        trans.setEndpoint(endpoint);

        return trans;
    }

    public Object getTestData()
    {
        if (message == null)
        {
            message = new MimeMessage(Session.getDefaultInstance(new Properties()));
            try
            {
                message.setContent(getResultData(), "text/plain");
            } catch (MessagingException e)
            {
                throw new RuntimeException("Failed to create Mime Message: " + e.getMessage(), e);
            }
        }
        return message;
    }

    public Object getResultData()
    {
        return "Test Email Message";
    }

    public boolean compareResults(Object src, Object result)
    {
        if(src instanceof Message) {
            Object objSrc = null;
            Object objRes = null;
            try
            {
                objSrc = ((Message)src).getContent();
                objRes = ((Message)result).getContent();
            } catch (Exception e)
            {
                throw new RuntimeException(e.getMessage(), e);
            }
            if(objSrc==null || objRes==null) return false;
            return objRes.equals(objSrc);
        }
        return super.compareResults(src, result);
    }
}
